/*****************************************************************************/
/* FE->BE transactions for the BLK2 split driver interface.                  */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef BLKIF2_H
#define BLKIF2_H

#include <asm-xen/xenidc.h>

typedef u64 blkif2_lba;

struct blkif2_transaction_parameters_header {
	u8 transaction_type;
	u8 reserved[7];
};

#define BLKIF2_TRANSACTION_TYPE_IO 0

struct blkif2_io_transaction_parameters {
	struct blkif2_transaction_parameters_header header;
	u8                                          io_type;
	u8                                          reserved[7];
	blkif2_lba                                  lba;
	struct xenidc_rbr                           rbr;
};

#define BLKIF2_IO_TYPE_READ  0
#define BLKIF2_IO_TYPE_WRITE 1

static inline xenidc_error blkif2_error_map_local_to(int error)
{
	switch (error) {
	default:
		return xenidc_error_map_local_to(error);
	}
}

static inline int blkif2_error_map_to_local(xenidc_error error)
{
	switch (error) {
	default:
		return xenidc_error_map_to_local(error);
	}
}

union blkif2_max_transaction {
	struct {
		struct blkif2_io_transaction_parameters parameters;
	} io;
};

#define BLKIF2_MAX_BULK_BYTE_COUNT 65536 /* FIXME: what value here? */
#define BLKIF2_BULK_BYTE_ALIGNMENT 1	 /* FIXME: what value here? */

#define BLKIF2_BE_INITIATOR_QUOTA 0
#define BLKIF2_BE_INITIATOR_MAXIMUM_BYTE_COUNT 0

#define BLKIF2_BE_TARGET_QUOTA 32
#define BLKIF2_BE_TARGET_MAXIMUM_BYTE_COUNT \
( sizeof( union blkif2_max_transaction ) )

#define BLKIF2_FE_INITIATOR_QUOTA BLKIF2_BE_TARGET_QUOTA
#define BLKIF2_FE_INITIATOR_MAXIMUM_BYTE_COUNT \
BLKIF2_BE_TARGET_MAXIMUM_BYTE_COUNT

#define BLKIF2_FE_TARGET_QUOTA BLKIF2_BE_INITIATOR_QUOTA
#define BLKIF2_FE_TARGET_MAXIMUM_BYTE_COUNT \
BLKIF2_BE_INITIATOR_MAXIMUM_BYTE_COUNT

#endif
